/* 
IR Receiver
by Natasha Owczarek at Tomorrow Lab, 2012
receive IR pulse of morse code on a regular (non modulating) npn phototransistor
and translate to letters using timing protocol set for IR emitter/receiver pair
*/


const int detectPin = A0;  // connect output (collector) of phototransistor
const int ledPin = 11;    //connect morse LED pin

int detect = 0;        // value detected at phototransistor
int ambient = 0;
int freq = 255;          // frequency of pwm signal (0-255 goes up to ~490 hz)
int space = 300;

boolean pulse = 0;              // to detect new change in light level
boolean prevPulse = 0;
int timeON = 0;                // to measure duration of on and off cycles  
unsigned long timeOFF = 0; 
unsigned long beginON = 0;
unsigned long beginOFF = 0;
int threshold = 5;              // threshold for IR light on/off. increase/decrease as necessary

String letter, morseLetter;
boolean debug = 0;              // set to 1 for debugging data on serial

void setup() {
  Serial.begin(9600);
  Serial.println(" starting... ");
  ambient = analogRead(detectPin);    // find out ambient light level
  Serial.print("ambient light level reading (in bits): ");
  Serial.println(ambient);
}

void loop() {

  
  //receive pulses
  detect = analogRead(detectPin);
  if(debug){Serial.print("detected: ");}
  if(debug){Serial.println(detect);}
  int diff = detect - ambient;
  
  //turn on LED if high
  if (diff > threshold) {
    digitalWrite(ledPin, HIGH);
  }
  else {
    digitalWrite(ledPin, LOW);
  }
  if(debug){Serial.print("difference is ");}
  if(debug){Serial.println(diff);}
  
  //timing on/off cycles
  if ((diff > threshold) && prevPulse == 0) {                  //emitter 'on'
    if(debug){Serial.println("emitter is ON");}
    pulse = 1;
    prevPulse = pulse;
    beginON = millis();
    if (beginOFF > 0) {                  //if not first time on
      timeOFF = beginON - beginOFF;
      if (timeOFF > 300){
        Serial.print("reading the letter...      ");
        letter = readLetter();
        Serial.println(letter);  
        letter = "";
      }
    }
  }
  if ((detect - ambient < threshold) && prevPulse == 1) {      //emitter 'off'
    if(debug){Serial.println("emitter turned OFF");}
    pulse = 0;
    prevPulse = pulse;
    beginOFF = millis();
    timeON = beginOFF - beginON;
    if (timeON > 200) {
      //strcat(morseLetter, "l");                  //add a dash
      morseLetter += 'l';
      if(debug){Serial.println(" adding a long... ");}
  
    }    
    else {if (timeON > 50) {
      //strcat(morseLetter, "s");                  //add a dot
      morseLetter += 's';
      if(debug){Serial.println(" adding a short... ");}
    }
    }    
  }
  else if (beginOFF > 0 && (millis() - beginOFF) > 1000 && prevPulse == 0) {     //time out to send last letter and clear strings
    if(debug){Serial.println("....in timeout clause");}
    if(debug){Serial.print("reading the last letter...      ");}
    letter = readLetter();
    Serial.println(letter);  
    letter = "";
    ambient = analogRead(detectPin);          //check and reset ambient when low... ?
    pulse = 0;
    prevPulse = 0;
  }
}



////////////////////////////   READING LETTER FROM MORSE CODE /////////////////////////////
String readLetter() {
  //determine letter from morse code
  if (morseLetter.charAt(0) == 's') {            // .  (starting with dot)
  //if (morseLetter == "sss") {
    //return 'S';
      //}

    if (morseLetter.charAt(1) == 's') {          // ..
      if (morseLetter.charAt(2) == 's') {        // ...
        if (morseLetter == "sss") {
        letter += "S";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "sssl") {
        letter += "V";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "ssss") {
        letter += "H";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "sssll") {
        letter += "3";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "ssssl") {
        letter += "4";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "sssss") {
        letter += "5";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      }
      else if (morseLetter.charAt(2) == 'l') {    //  ..-
        if (morseLetter == "ssl") {
        letter += "U";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "ssls") {
        letter += "F";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "sslll") {
        letter += "2";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      }
      else if (morseLetter == "ss") {
        letter += "I";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    
    else if (morseLetter.charAt(1) == 'l') {    // .-
     if (morseLetter.charAt(2) == 's') {        // .-.
      if (morseLetter == "sls") {
        letter += "R";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "slss") {
        letter += "L";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    else if (morseLetter.charAt(2) == 'l') {    // .--
      if (morseLetter == "sll") {
        letter += "W";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "slls") {
        letter += "P";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "slll") {
        letter += "J";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "sllll") {
        letter += "1";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    else if (morseLetter == "sl") {
        letter += "A";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    if (morseLetter == "s") {
        letter += "E";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
  }
  else if (morseLetter.charAt(0) == 'l') {    //  -  (starting with dash)
    if (morseLetter.charAt(1) == 's') {          // -.
      if (morseLetter.charAt(2) == 's') {        // -..
        if (morseLetter == "lss") {
        letter += "D";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lssl") {
        letter += "X";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lsss") {
        letter += "B";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lssss") {
        letter += "6";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      }
      else if (morseLetter.charAt(2) == 'l') {    //  -.-
        if (morseLetter == "lsl") {
        letter += "K";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lsls") {
        letter += "C";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lsll") {
        letter += "Y";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      }
      else if (morseLetter == "ls") {
        letter += "N";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    
    else if (morseLetter.charAt(1) == 'l') {    // --
     if (morseLetter.charAt(2) == 's') {        // --.
      if (morseLetter == "lls") {
        letter += "G";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "llss") {
        letter += "Z";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "llsl") {
        letter += "Q";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "llsss") {
        letter += "7";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    else if (morseLetter.charAt(2) == 'l') {    // ---
      if (morseLetter == "lll") {
        letter += "O";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lllss") {
        letter += "8";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lllls") {
        letter += "9";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
      if (morseLetter == "lllll") {
        letter += "0";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    else if (morseLetter == "ll") {
        letter += "M";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }
    }
    if (morseLetter == "l") {
        letter += "T";
        if(debug){Serial.println(letter);}
        morseLetter = "";
        return letter;
      }    
  }
  else {
    morseLetter = "";
    return "";
}
}
